@@ -325,7 +325,7 @@ create_popup (GpmBrightnessApplet *applet)
 	g_signal_connect (G_OBJECT(applet->slider), "value-changed", G_CALLBACK(slide_cb), applet);
 
 	/* minus button */
-	applet->btn_minus = gtk_button_new_with_label ("-");
+	applet->btn_minus = gtk_button_new_with_label ("\342\210\222"); /* U+2212 MINUS SIGN */
 	gtk_button_set_relief (GTK_BUTTON(applet->btn_minus), GTK_RELIEF_NONE);
 	g_signal_connect (G_OBJECT(applet->btn_minus), "pressed", G_CALLBACK(minus_cb), applet);
 
@@ -535,7 +535,11 @@ help_cb (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 {
 	GError *error = NULL;
 	GpmBrightnessApplet *applet = GPM_BRIGHTNESS_APPLET(data);
-	gnome_help_display ("gnome-power-icon.xml", NULL, &error);
+	GnomeProgram *program = gnome_program_get ();
+
+	gnome_help_display_with_doc_id (program, "gnome-power-manager",
+					"gnome-power-manager.xml",
+					"applets-brightness", &error);
 	if (error != NULL) {
 		GtkWidget *dialog =
 			gtk_message_dialog_new (GTK_WINDOW (GTK_WIDGET(applet)->parent),
