@@ -26,13 +26,16 @@
 
 G_BEGIN_DECLS
 
-#define	HAL_DBUS_SERVICE		"org.freedesktop.Hal"
-#define	HAL_DBUS_PATH_MANAGER		"/org/freedesktop/Hal/Manager"
-#define	HAL_DBUS_INTERFACE_MANAGER	"org.freedesktop.Hal.Manager"
-#define	HAL_DBUS_INTERFACE_DEVICE	"org.freedesktop.Hal.Device"
-#define	HAL_DBUS_INTERFACE_LAPTOP_PANEL	"org.freedesktop.Hal.Device.LaptopPanel"
-#define	HAL_DBUS_INTERFACE_POWER	"org.freedesktop.Hal.Device.SystemPowerManagement"
-#define HAL_ROOT_COMPUTER		"/org/freedesktop/Hal/devices/computer"
+#define	HAL_DBUS_SERVICE		 "org.freedesktop.Hal"
+#define	HAL_DBUS_PATH_MANAGER		 "/org/freedesktop/Hal/Manager"
+#define	HAL_DBUS_INTERFACE_MANAGER	 "org.freedesktop.Hal.Manager"
+#define	HAL_DBUS_INTERFACE_DEVICE	 "org.freedesktop.Hal.Device"
+#define	HAL_DBUS_INTERFACE_LAPTOP_PANEL	 "org.freedesktop.Hal.Device.LaptopPanel"
+#define	HAL_DBUS_INTERFACE_POWER	 "org.freedesktop.Hal.Device.SystemPowerManagement"
+#define	HAL_DBUS_INTERFACE_CPUFREQ	 "org.freedesktop.Hal.Device.CPUFreq"
+#define	HAL_DBUS_INTERFACE_KBD_BACKLIGHT "org.freedesktop.Hal.Device.KeyboardBacklight"
+#define	HAL_DBUS_INTERFACE_LIGHT_SENSOR	 "org.freedesktop.Hal.Device.LightSensor"
+#define HAL_ROOT_COMPUTER		 "/org/freedesktop/Hal/devices/computer"
 
 #define GPM_TYPE_HAL		(gpm_hal_get_type ())
 #define GPM_HAL(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), GPM_TYPE_HAL, GpmHal))
@@ -65,25 +68,25 @@ typedef struct
 {
 	GObjectClass	parent_class;
 	void		(* device_added)		(GpmHal		*hal,
-							 const char	*udi);
+							 const gchar	*udi);
 	void		(* device_removed)		(GpmHal		*hal,
-							 const char	*udi);
+							 const gchar	*udi);
 	void		(* device_property_modified)	(GpmHal		*hal,
-							 const char	*udi,
-							 const char	*key,
+							 const gchar	*udi,
+							 const gchar	*key,
 							 gboolean	 is_added,
 							 gboolean	 is_removed,
 							 gboolean	 finally);
 	void		(* device_condition)		(GpmHal		*hal,
-							 const char	*udi,
-							 const char	*condition,
-							 const char	*details);
+							 const gchar	*udi,
+							 const gchar	*condition,
+							 const gchar	*details);
 	void		(* new_capability)		(GpmHal		*hal,
-							 const char	*udi,
-							 const char	*capability);
+							 const gchar	*udi,
+							 const gchar	*capability);
 	void		(* lost_capability)		(GpmHal		*hal,
-							 const char	*udi,
-							 const char	*capability);
+							 const gchar	*udi,
+							 const gchar	*capability);
 	void		(* daemon_start)		(GpmHal		*hal);
 	void		(* daemon_stop)			(GpmHal		*hal);
 } GpmHalClass;
@@ -92,19 +95,7 @@ GType		 gpm_hal_get_type			(void);
 GpmHal		*gpm_hal_new				(void);
 gboolean	 gpm_hal_get_idle			(GpmHal		*hal);
 
-gboolean	 gpm_hal_has_power_management		(GpmHal		*hal);
 gboolean	 gpm_hal_is_running			(GpmHal		*hal);
-gboolean	 gpm_hal_is_laptop			(GpmHal		*hal);
-gboolean	 gpm_hal_is_on_ac			(GpmHal		*hal);
-gboolean	 gpm_hal_can_suspend			(GpmHal		*hal);
-gboolean	 gpm_hal_suspend			(GpmHal		*hal,
-							 gint		 wakeup);
-gboolean	 gpm_hal_can_hibernate			(GpmHal		*hal);
-gboolean	 gpm_hal_hibernate			(GpmHal		*hal);
-gboolean	 gpm_hal_shutdown			(GpmHal		*hal);
-gboolean	 gpm_hal_reboot				(GpmHal		*hal);
-gboolean	 gpm_hal_enable_power_save		(GpmHal		*hal,
-							 gboolean	 enable);
 gboolean	 gpm_hal_device_get_bool		(GpmHal		*hal,
 							 const gchar	*udi,
 							 const gchar	*key,
@@ -117,11 +108,19 @@ gboolean	 gpm_hal_device_get_int			(GpmHal		*hal,
 							 const gchar	*udi,
 							 const gchar	*key,
 							 gint		*value);
+gboolean	 gpm_hal_device_get_uint		(GpmHal		*hal,
+							 const gchar	*udi,
+							 const gchar	*key,
+							 guint		*value);
 gboolean	 gpm_hal_device_find_capability		(GpmHal		*hal,
 							 const gchar	*capability,
 							 gchar	      ***value);
 gboolean	 gpm_hal_device_rescan_capability	(GpmHal		*hal,
 							 const char	*capability);
+gboolean	 gpm_hal_device_has_capability		(GpmHal		*hal,
+							 const gchar	*udi,
+							 const gchar	*capability,
+							 gboolean	*has_capability);
 gint		 gpm_hal_num_devices_of_capability	(GpmHal		*hal,
 							 const gchar	*capability);
 gint		 gpm_hal_num_devices_of_capability_with_value (GpmHal	*hal,
@@ -131,14 +130,34 @@ gint		 gpm_hal_num_devices_of_capability_with_value (GpmHal	*hal,
 void		 gpm_hal_free_capability		(GpmHal		*hal,
 							 gchar		**value);
 gboolean	 gpm_hal_device_watch_condition		(GpmHal		*hal,
-							 const char	*udi);
+							 const char	*udi,
+							 gboolean	 force);
 gboolean	 gpm_hal_device_watch_propery_modified	(GpmHal		*hal,
-							 const char	*udi);
+							 const char	*udi,
+							 gboolean	 force);
 gboolean	 gpm_hal_device_remove_condition	(GpmHal		*hal,
 							 const char	*udi);
 gboolean	 gpm_hal_device_remove_propery_modified	(GpmHal		*hal,
 							 const char	*udi);
 
+gboolean	 gpm_hal_has_power_management		(GpmHal		*hal);
+gboolean	 gpm_hal_can_suspend			(GpmHal		*hal);
+gboolean	 gpm_hal_suspend			(GpmHal		*hal,
+							 guint		 wakeup);
+gboolean	 gpm_hal_can_hibernate			(GpmHal		*hal);
+gboolean	 gpm_hal_hibernate			(GpmHal		*hal);
+gboolean	 gpm_hal_shutdown			(GpmHal		*hal);
+gboolean	 gpm_hal_reboot				(GpmHal		*hal);
+gboolean	 gpm_hal_enable_power_save		(GpmHal		*hal,
+							 gboolean	 enable);
+gboolean	 gpm_hal_is_laptop			(GpmHal		*hal);
+gboolean	 gpm_hal_has_suspend_error		(GpmHal		*hal,
+							 gboolean	*state);
+gboolean	 gpm_hal_has_hibernate_error		(GpmHal		*hal,
+							 gboolean	*state);
+gboolean	 gpm_hal_clear_suspend_error		(GpmHal		*hal);
+gboolean	 gpm_hal_clear_hibernate_error		(GpmHal		*hal);
+
 G_END_DECLS
 
 #endif	/* __GPMHAL_H */
