@@ -34,6 +34,7 @@
 #include "gpm-hal.h"
 #include "gpm-battery.h"
 
+#include "gpm-control.h"
 #include "gpm-power.h"
 #include "gpm-marshal.h"
 #include "gpm-refcount.h"
@@ -57,6 +58,7 @@ struct GpmPowerPrivate
 	GHashTable		*battery_kind_cache;
 	GHashTable		*battery_device_cache;
 	GpmBattery		*battery;
+	GpmControl		*control;
 	GpmAcAdapter		*ac_adapter;
 	GpmConf			*conf;
 	GpmHal			*hal;
@@ -520,15 +522,15 @@ gpm_power_kind_to_string (GpmPowerKind battery_kind)
 	if (battery_kind == GPM_POWER_KIND_PRIMARY) {
  		str = "primary";
 	} else if (battery_kind == GPM_POWER_KIND_UPS) {
- 		str = _("ups");
+ 		str = "ups";
 	} else if (battery_kind == GPM_POWER_KIND_MOUSE) {
- 		str = _("mouse");
+ 		str = "mouse";
 	} else if (battery_kind == GPM_POWER_KIND_KEYBOARD) {
- 		str = _("keyboard");
+ 		str = "keyboard";
 	} else if (battery_kind == GPM_POWER_KIND_PDA) {
- 		str = _("pda");
+ 		str = "pda";
  	} else {
- 		str = _("unknown");
+ 		str = "unknown";
 	}
 	return str;
 }
@@ -1071,11 +1073,11 @@ battery_kind_cache_update (GpmPower		 *power,
 	if( type_status->charge_rate_smoothed != 0 ) {
 		type_status->charge_rate_previous = type_status->charge_rate_smoothed;
 	} else {
-		gpm_warning("Exponential average of 0 calculated (not okay.):"
-			    " inputs were charge_rate_previous=%d, charge_rate_raw=%d, exp_ave_factor=%d",
-			    type_status->charge_rate_previous,
-			    type_status->charge_rate_raw,
-			    power->priv->exp_ave_factor );
+		gpm_debug ("Exponential average of 0 calculated:"
+			   " inputs were charge_rate_previous=%d, charge_rate_raw=%d, exp_ave_factor=%d",
+			   type_status->charge_rate_previous,
+			   type_status->charge_rate_raw,
+			   power->priv->exp_ave_factor );
  	}
 
 	/* average out the voltage for the global device */
@@ -1814,6 +1816,21 @@ hal_daemon_stop_cb (GpmHal   *hal,
 	gpm_hash_free_device_cache (power);
 }
 
+/**
+ * control_resume_cb:
+ * @control: The control class instance
+ * @power: This power class instance
+ *
+ * We have to update the caches on resume
+ **/
+static void
+control_resume_cb (GpmControl *control,
+		   GpmControlAction action,
+		   GpmPower   *power)
+{
+	gpm_power_update_all (power);
+}
+
 /**
  * gpm_power_init:
  * @power: This power class instance
@@ -1853,6 +1870,10 @@ gpm_power_init (GpmPower *power)
 	g_signal_connect (power->priv->refcount, "refcount-added",
 			  G_CALLBACK (gpm_power_refcount_added), power);
 
+	power->priv->control = gpm_control_new ();
+	g_signal_connect (power->priv->control, "resume",
+			  G_CALLBACK (control_resume_cb), power);
+
 	/* we get this from gconf as some machines take longer to settle down */
 	gpm_conf_get_uint (power->priv->conf, GPM_CONF_INVALID_TIMEOUT, &invalid_timeout);
 	gpm_refcount_set_timeout (power->priv->refcount, invalid_timeout);
@@ -1890,6 +1911,9 @@ gpm_power_finalize (GObject *object)
 
 	g_object_unref (power->priv->conf);
 
+	if (power->priv->control != NULL) {
+		g_object_unref (power->priv->control);
+	}
 	if (power->priv->hal != NULL) {
 		g_object_unref (power->priv->hal);
 	}
