@@ -45,36 +45,31 @@ typedef struct
 typedef struct
 {
 	GObjectClass	parent_class;
-	void	(* suspend)			(GpmTrayIcon *tray_icon);
-	void	(* hibernate)			(GpmTrayIcon *tray_icon);
+	void	(* suspend)				(GpmTrayIcon	*tray_icon);
+	void	(* hibernate)				(GpmTrayIcon	*tray_icon);
+	void	(* description_changed)			(GpmTrayIcon	*tray_icon,
+							 const gchar	**description);
+	void	(* icon_changed)			(GpmTrayIcon	*tray_icon,
+							 const gchar	**icon);
 } GpmTrayIconClass;
 
-typedef enum {
-	GPM_NOTIFY_URGENCY_CRITICAL = 1,
-	GPM_NOTIFY_URGENCY_NORMAL   = 2,
-	GPM_NOTIFY_URGENCY_LOW      = 3
-} GpmNotifyLevel;
-
 GType		 gpm_tray_icon_get_type			(void);
 GpmTrayIcon	*gpm_tray_icon_new			(void);
 
-void		 gpm_tray_icon_notify			(GpmTrayIcon	*icon,
-							 const gchar	*title,
-							 const gchar	*content,
-							 guint		 timeout,
-							 const gchar	*msgicon,
-							 GpmNotifyLevel	 urgency);
-void		 gpm_tray_icon_cancel_notify		(GpmTrayIcon	*icon);
 void		 gpm_tray_icon_set_tooltip		(GpmTrayIcon	*icon,
 							 const gchar	*tooltip);
 void		 gpm_tray_icon_set_image_from_stock	(GpmTrayIcon	*icon,
 							 const gchar	*stock_id);
-void		 gpm_tray_icon_enable_suspend		(GpmTrayIcon	*icon,
-							 gboolean	 enabled);
-void		 gpm_tray_icon_enable_hibernate		(GpmTrayIcon	*icon,
-							 gboolean	 enabled);
-void		 gpm_tray_icon_show			(GpmTrayIcon	*icon,
-							 gboolean	 enabled);
+void		 gpm_tray_icon_sync			(GpmTrayIcon	*icon);
+
+gboolean
+gpm_ui_get_description (GpmTrayIcon  *icon,
+		        gchar **mode_str,
+		        GError      **error);
+gboolean
+gpm_ui_get_icon (GpmTrayIcon  *icon,
+		 gchar **mode_str,
+		 GError      **error);
 
 G_END_DECLS
 
